# $Id: Movie.pm 3737 2010-01-19 12:25:39Z chris $

=head1 NAME

WebService::RottenTomatoes::Movie

=head1 SYNOPSIS

=head1 DESCRIPTION

=cut

package WebService::RottenTomatoes::Movie;

use strict;
use warnings;

our $VERSION = '0.01';

use base qw(Class::Accessor WebService::RottenTomatoes::Base);

use Carp;
our @CARP_NOT = qw(WebService::RottenTomatoes);

use HTML::TokeParser;

use URI;

__PACKAGE__->mk_accessors(qw(
    _ws

    rtname

    _parsed_scores
    _tmeter_critics_score
    _top_critics_score
    _rt_community_score
    _my_critics_scores
    _my_friends_score
    _dvd_score
    _tmeter_critics_details
));

sub _new {
    my $class = shift;
    my $ws = shift;
    my %args = @_;

    my $self = {};

    bless $self, $class;

    $self->_ws($ws);

    if ($args{'rtname'}) {
	$self->rtname($args{'rtname'});
    } else {
	my $uri = URI->new();
	$uri->scheme("http");
	$uri->host($self->_ws()->domain());
	if ($args{'rtid'}) {
	    $uri->path(sprintf("m/Movie-%d/", $args{'rtid'}));
	} elsif ($args{'imdbid'}) {
	    my ($imdb_code) = $args{'imdbid'} =~ m/^tt(\d{6,7})$/ or croak "Unexpected format for imdbid '$args{'imdbid'}'";
	    $uri->path("alias");
	    $uri->query_form("type" => "imdbid", "s" => sprintf("%07d", $imdb_code));
	} elsif ($args{'imdbtitle'}) {
	    $uri->path("alias");
	    $uri->query_form("s" => $args{'imdbtitle'});
	} else {
	    croak "No valid search criteria";
	}

	my $response = $self->_ws()->_get_page($uri->as_string());
	my $redirectCount = $response->redirects();

	if ($redirectCount == 0) {
	    die "redirectCount == 0";
	}

	my $redirectUrl = ($response->redirects())[$redirectCount - 1]->header("Location");

	$redirectUrl =~ m/\/movie\/$/ and croak "Resource not found";
	my ($rtname) = $redirectUrl =~ m/\/m\/([a-z_]+)\/$/ or die "Unable to extract rtname from '$redirectUrl'";
	$self->rtname($rtname);
    }


    return $self;
}

sub _url {
    my $self = shift;
    my $page = shift;

    my $uri = URI->new();
    $uri->scheme("http");
    $uri->host($self->_ws->domain());
    $uri->path("m/" . $self->rtname() . "/" . ((defined $page) ? $page : ""));

    return $uri->as_string();

}

=head1 METHODS

=head2 rtname

=head2 tmeter_critics_score

=head2 top_critics_score

=head2 rt_community_score

=head2 my_critics_scores

=head2 my_friends_score

=head2 dvd_score

=cut

sub tmeter_critics_score {
    my $self = shift;
    $self->_parse_scores();
    return $self->_tmeter_critics_score()
}

sub top_critics_score {
    my $self = shift;
    $self->_parse_scores();
    return $self->_top_critics_score();
}

sub rt_community_score {
    my $self = shift;
    $self->_parse_scores();
    return $self->_rt_community_score();
}

sub my_critics_scores {
    my $self = shift;
    $self->_parse_scores();
    return $self->_my_critics_scores();
}

sub my_friends_score {
    my $self = shift;
    $self->_parse_scores();
    return $self->_my_friends_score();
}

sub dvd_score {
    my $self = shift;
    $self->_parse_scores();
    return $self->_dvd_score();
}


sub tmeter_critics_details {
    my $self = shift;

    my $details = $self->_tmeter_critics_details();
    if (!defined $details) {
	$details = $self->_parse_score_details(\$self->_content());
	$self->_tmeter_critics_details($details);
    }

    return $details;
}



# Currently, the scores for each category come from the main page, using the title text
# for the link to the other score pages.  This seems a bit delicate, and it would perhaps
# be better just to fetch the other pages and extract the score from there.
sub _parse_scores {
    my $self = shift;

    if ($self->_parsed_scores()) { return; }
    $self->_parsed_scores(1);

    my $p = HTML::TokeParser->new( \$self->_content() );

    my $token;

    while ($token = $p->get_tag("ul")) {
	if ($token->[1]{'id'} eq "tomatometer_nav") { last; }
    }

    if (!defined $token) { die "Failed to get tomatometer_nav ul"; }

    while ($token = $p->get_tag("li", "/ul")) {
	if ($token->[0] eq "/ul") { last; }

	$token = $p->get_tag("a");

	my $text = $p->get_trimmed_text("/a");
	my $title = $token->[1]{'title'} || "";
	my $href = $token->[1]{'href'};
	my $score;

	if ($title eq "N/A") {
	    $score = undef;
	} elsif (($score) = $title =~ m/^(\d+)\%$/) {
	    $score += 0;
	} else {
	    die "Unable to parse score from '$title' $text $href";
	}

	if ($text eq "T-Meter Critics" && $href =~ m/\?name_order=asc$/) {
	    $self->_tmeter_critics_score($score);

	} elsif ($text eq "Top Critics" && $href =~ m/\?critic=creamcrop$/) {
	    $self->_top_critics_score($score);

	} elsif ($text eq "RT Community" && $href =~ m/^reviews_users\.php$/) {
	    $self->_rt_community_score($score);

	} elsif ($text eq "My Critics" && $href =~ m/^reviews_mycritics\.php$/) {
	    $self->_my_critics_scores($score);

	} elsif ($text eq "My Friends" && $href =~ m/^reviews_friends\.php$/) {
	    $self->_my_friends_score($score);

	} elsif ($text eq "DVD" && $href =~ m/\?critic=\d+$/) {
	    $self->_dvd_score($score);

	} else {
	    die "Unable to identify score type from '$text' '$href'";
	}
    }  
}
    
sub _parse_score_details {
    my $self = shift;
    my $content = shift;

    my $details = {};

    my $p = HTML::TokeParser->new( \$self->_content() );

    my $token;

    while ($token = $p->get_tag("div")) {
	if (defined $token->[1]{'id'} && $token->[1]{'id'} eq "tomatometer_data") { last; }
    }

    if (!defined $token) { die "Failed to get tomatometer_data div"; }

    while ($token = $p->get_tag("p", "/div")) {
	if ($token->[0] eq "/div") { last; }

	my $text = $p->get_trimmed_text("/p");

	my ($n, $v) = $text =~ m/^([\w ]+)\s*:\s*([\d.\/]+)$/ or die "Failed to parse '$text'";

	if ($n eq "Reviews Counted") {
	    $details->{'total'} = $v + 0;
	} elsif ($n eq "Fresh") {
	    $details->{'fresh'} = $v + 0;
	} elsif ($n eq "Rotten") {
	    $details->{'rotten'} = $v + 0;
	} elsif ($n eq "Average Rating") {
	    my ($num, $den) = $v =~ m/(\d+(?:.\d+)?)\/(\d+(?:.\d+)?)/ or die "Failed to parse '$v'";
	    $details->{'average'} = [$num, $den];
	} else {
	    die "Don't recognise key '$n'";
	}
    }

    return $details;

}

1;
